@@ -24,7 +24,7 @@ from octaviaclient.tests.unit.osc.v2 import fakes
 class TestListener(fakes.TestOctaviaClient):
 
     def setUp(self):
-        super(TestListener, self).setUp()
+        super().setUp()
 
         self._listener = fakes.createFakeResource('listener')
         self.listener_info = copy.deepcopy(attr_consts.LISTENER_ATTRS)
@@ -40,7 +40,7 @@ class TestListener(fakes.TestOctaviaClient):
 class TestListenerList(TestListener):
 
     def setUp(self):
-        super(TestListenerList, self).setUp()
+        super().setUp()
         self.datalist = (tuple(
             attr_consts.LISTENER_ATTRS[k] for k in self.columns),)
         self.cmd = listener.ListListener(self.app, None)
@@ -67,11 +67,65 @@ class TestListenerList(TestListener):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, tuple(data))
 
+    def test_listener_list_with_tags(self):
+        arglist = ['--tags', 'foo,bar']
+        verifylist = [('tags', ['foo', 'bar'])]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+        self.api_mock.listener_list.assert_called_with(
+            tags=['foo', 'bar'])
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
+
+    def test_listener_list_with_any_tags(self):
+        arglist = ['--any-tags', 'foo,bar']
+        verifylist = [('any_tags', ['foo', 'bar'])]
+        expected_attrs = {
+            "tags-any": ['foo', 'bar']
+        }
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+        self.api_mock.listener_list.assert_called_with(**expected_attrs)
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
+
+    def test_listener_list_with_not_tags(self):
+        arglist = ['--not-tags', 'foo,bar']
+        verifylist = [('not_tags', ['foo', 'bar'])]
+        expected_attrs = {
+            "not-tags": ['foo', 'bar']
+        }
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+        self.api_mock.listener_list.assert_called_with(**expected_attrs)
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
+
+    def test_listener_list_with_not_any_tags(self):
+        arglist = ['--not-any-tags', 'foo,bar']
+        verifylist = [('not_any_tags', ['foo', 'bar'])]
+        expected_attrs = {
+            "not-tags-any": ['foo', 'bar']
+        }
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+        self.api_mock.listener_list.assert_called_with(**expected_attrs)
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, tuple(data))
+
 
 class TestListenerDelete(TestListener):
 
     def setUp(self):
-        super(TestListenerDelete, self).setUp()
+        super().setUp()
         self.cmd = listener.DeleteListener(self.app, None)
 
     def test_listener_delete(self):
@@ -108,6 +162,7 @@ class TestListenerDelete(TestListener):
         verifylist = [
             ('listener', 'unknown_lb')
         ]
+        self.api_mock.listener_list.return_value = {'listeners': []}
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         self.assertRaises(exceptions.CommandError, self.cmd.take_action,
                           parsed_args)
@@ -117,7 +172,7 @@ class TestListenerDelete(TestListener):
 class TestListenerCreate(TestListener):
 
     def setUp(self):
-        super(TestListenerCreate, self).setUp()
+        super().setUp()
         self.api_mock.listener_create.return_value = {
             'listener': self.listener_info}
         lb_client = self.app.client_manager
@@ -196,7 +251,11 @@ class TestListenerCreate(TestListener):
                    '--tls-version',
                    self._listener.tls_versions[0],
                    '--tls-version',
-                   self._listener.tls_versions[1]]
+                   self._listener.tls_versions[1],
+                   '--alpn-protocol',
+                   self._listener.alpn_protocols[0],
+                   '--alpn-protocol',
+                   self._listener.alpn_protocols[1]]
 
         verifylist = [
             ('loadbalancer', 'mock_lb_id'),
@@ -214,7 +273,9 @@ class TestListenerCreate(TestListener):
             ('tls_ciphers',
              self._listener.tls_ciphers),
             ('tls_versions',
-             self._listener.tls_versions)
+             self._listener.tls_versions),
+            ('alpn_protocols',
+             self._listener.alpn_protocols),
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -249,11 +310,32 @@ class TestListenerCreate(TestListener):
         self.api_mock.listener_create.assert_called_with(
             json={'listener': self.listener_info})
 
+    @mock.patch('octaviaclient.osc.v2.utils.get_listener_attrs')
+    def test_listener_create_with_tag(self, mock_client):
+        mock_client.return_value = self.listener_info
+        arglist = ['mock_lb_id',
+                   '--name', self._listener.name,
+                   '--protocol', 'HTTP',
+                   '--protocol-port', '80',
+                   '--tag', 'foo']
+        verifylist = [
+            ('loadbalancer', 'mock_lb_id'),
+            ('name', self._listener.name),
+            ('protocol', 'HTTP'),
+            ('protocol_port', 80),
+            ('tags', ['foo']),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+        self.api_mock.listener_create.assert_called_with(
+            json={'listener': self.listener_info})
+
 
 class TestListenerShow(TestListener):
 
     def setUp(self):
-        super(TestListenerShow, self).setUp()
+        super().setUp()
         self.api_mock.listener_show.return_value = self.listener_info
         lb_client = self.app.client_manager
         lb_client.load_balancer = self.api_mock
@@ -275,7 +357,7 @@ class TestListenerShow(TestListener):
 class TestListenerSet(TestListener):
 
     def setUp(self):
-        super(TestListenerSet, self).setUp()
+        super().setUp()
         self.cmd = listener.SetListener(self.app, None)
 
     def test_listener_set(self):
@@ -300,7 +382,11 @@ class TestListenerSet(TestListener):
                    '--tls-version',
                    self._listener.tls_versions[0],
                    '--tls-version',
-                   self._listener.tls_versions[1]]
+                   self._listener.tls_versions[1],
+                   '--alpn-protocol',
+                   self._listener.alpn_protocols[0],
+                   '--alpn-protocol',
+                   self._listener.alpn_protocols[1]]
         verifylist = [
             ('listener', self._listener.id),
             ('name', 'new_name'),
@@ -315,7 +401,8 @@ class TestListenerSet(TestListener):
              self._listener.client_crl_container_ref),
             ('allowed_cidrs', self._listener.allowed_cidrs),
             ('tls_ciphers', self._listener.tls_ciphers),
-            ('tls_versions', self._listener.tls_versions)
+            ('tls_versions', self._listener.tls_versions),
+            ('alpn_protocols', self._listener.alpn_protocols)
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -336,6 +423,7 @@ class TestListenerSet(TestListener):
                     'allowed_cidrs': self._listener.allowed_cidrs,
                     'tls_ciphers': self._listener.tls_ciphers,
                     'tls_versions': self._listener.tls_versions,
+                    'alpn_protocols': self._listener.alpn_protocols,
                 }})
 
     @mock.patch('osc_lib.utils.wait_for_status')
@@ -357,11 +445,49 @@ class TestListenerSet(TestListener):
             sleep_time=mock.ANY,
             status_field='provisioning_status')
 
+    def test_listener_set_tag(self):
+        self.api_mock.listener_show.return_value = {
+            'tags': ['foo']
+        }
+        arglist = [self._listener.id, '--tag', 'bar']
+        verifylist = [
+            ('listener', self._listener.id),
+            ('tags', ['bar']),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.api_mock.listener_set.assert_called_once()
+        kwargs = self.api_mock.listener_set.mock_calls[0][2]
+        tags = kwargs['json']['listener']['tags']
+        self.assertEqual(2, len(tags))
+        self.assertIn('foo', tags)
+        self.assertIn('bar', tags)
+
+    def test_listener_set_tag_no_tag(self):
+        self.api_mock.listener_show.return_value = {
+            'tags': ['foo']
+        }
+        arglist = [self._listener.id, '--tag', 'bar', '--no-tag']
+        verifylist = [
+            ('listener', self._listener.id),
+            ('tags', ['bar']),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.api_mock.listener_set.assert_called_once_with(
+            self._listener.id,
+            json={"listener": {"tags": ['bar']}}
+        )
+
 
 class TestListenerStatsShow(TestListener):
 
     def setUp(self):
-        super(TestListenerStatsShow, self).setUp()
+        super().setUp()
         listener_stats_info = {'stats': {'bytes_in': '0'}}
         self.api_mock.listener_stats_show.return_value = {
             'stats': listener_stats_info['stats']}
@@ -388,10 +514,10 @@ class TestListenerUnset(TestListener):
                   'timeout_member_connect', 'timeout_member_data',
                   'timeout_tcp_inspect', 'client_ca_tls_container_ref',
                   'client_authentication', 'client_crl_container_ref',
-                  'allowed_cidrs')
+                  'allowed_cidrs', 'tls_versions', 'tls_ciphers')
 
     def setUp(self):
-        super(TestListenerUnset, self).setUp()
+        super().setUp()
         self.cmd = listener.UnsetListener(self.app, None)
 
     def test_listener_unset_name(self):
@@ -442,6 +568,12 @@ class TestListenerUnset(TestListener):
     def test_listener_unset_allowed_cidrs(self):
         self._test_listener_unset_param('allowed_cidrs')
 
+    def test_listener_unset_tls_versions(self):
+        self._test_listener_unset_param('tls_versions')
+
+    def test_listener_unset_tls_ciphers(self):
+        self._test_listener_unset_param('tls_ciphers')
+
     def _test_listener_unset_param(self, param):
         self.api_mock.listener_set.reset_mock()
         arg_param = param.replace('_', '-') if '_' in param else param
@@ -511,3 +643,43 @@ class TestListenerUnset(TestListener):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         self.cmd.take_action(parsed_args)
         self.api_mock.listener_set.assert_not_called()
+
+    def test_listener_unset_tag(self):
+        self.api_mock.listener_set.reset_mock()
+        self.api_mock.listener_show.return_value = {
+            'tags': ['foo', 'bar']
+        }
+
+        arglist = [self._listener.id, '--tag', 'foo']
+        verifylist = [
+            ('listener', self._listener.id),
+            ('tags', ['foo']),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.api_mock.listener_set.assert_called_once_with(
+            self._listener.id,
+            json={"listener": {"tags": ['bar']}}
+        )
+
+    def test_listener_unset_all_tag(self):
+        self.api_mock.listener_set.reset_mock()
+        self.api_mock.listener_show.return_value = {
+            'tags': ['foo', 'bar']
+        }
+
+        arglist = [self._listener.id, '--all-tag']
+        verifylist = [
+            ('listener', self._listener.id),
+            ('all_tag', True),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.api_mock.listener_set.assert_called_once_with(
+            self._listener.id,
+            json={"listener": {"tags": []}}
+        )
