@@ -305,6 +305,8 @@ class TestLoadBalancerDelete(TestLoadBalancer):
         verifylist = [
             ('loadbalancer', 'unknown_lb')
         ]
+        self.api_mock.load_balancer_list.return_value = {
+            'loadbalancers': []}
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         self.assertRaises(exceptions.CommandError, self.cmd.take_action,
                           parsed_args)
@@ -396,6 +398,39 @@ class TestLoadBalancerCreate(TestLoadBalancer):
         self.api_mock.load_balancer_create.assert_called_with(
             json={'loadbalancer': lb_info})
 
+    @mock.patch('octaviaclient.osc.v2.utils.get_loadbalancer_attrs')
+    def test_load_balancer_create_with_additional_vips(self, mock_client):
+        mock_client.return_value = self.lb_info
+
+        arglist = [
+            '--name', self._lb.name,
+            '--vip-subnet-id', self._lb.vip_subnet_id,
+            '--project', self._lb.project_id,
+            '--additional-vip', 'subnet-id={},ip-address={}'.format(
+                self._lb.additional_vips[0]['subnet_id'],
+                self._lb.additional_vips[0]['ip_address']),
+            '--additional-vip', 'subnet-id={},ip-address={}'.format(
+                self._lb.additional_vips[1]['subnet_id'],
+                self._lb.additional_vips[1]['ip_address'])
+        ]
+        verifylist = [
+            ('name', self._lb.name),
+            ('vip_subnet_id', self._lb.vip_subnet_id),
+            ('project', self._lb.project_id),
+            ('additional_vip', [
+                'subnet-id={},ip-address={}'.format(
+                    self._lb.additional_vips[0]['subnet_id'],
+                    self._lb.additional_vips[0]['ip_address']),
+                'subnet-id={},ip-address={}'.format(
+                    self._lb.additional_vips[1]['subnet_id'],
+                    self._lb.additional_vips[1]['ip_address'])])
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+        self.api_mock.load_balancer_create.assert_called_with(
+            json={'loadbalancer': self.lb_info})
+
     @mock.patch('octaviaclient.osc.v2.utils.get_loadbalancer_attrs')
     def test_load_balancer_create_with_provider(self, mock_client):
         provider = 'foobar'
